//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

// updated February, 2014

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <GL/glut.h>

#include "Sprite.hh"

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

Sprite *sprite; 

float win_w = 250;
float win_h = 250;

float last_square_dx;
float square_dx = -1.0;

float square_x = 0.5 * win_w;
float square_y = 0.5 * win_h;

float square_theta = 0.0;

float square_scale = 1.0;

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------

void display(void)
{
  glClear (GL_COLOR_BUFFER_BIT);
  
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_BLEND);
  
  // x, y, theta (degrees)
  sprite->draw(square_x, square_y, square_theta,
	       square_scale * sprite->w, square_scale * sprite->h);
  
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
  
  glutSwapBuffers();
}

//----------------------------------------------------------------------------

void keyboard(char unsigned key, int x, int y)
{
  if (key == 'a') {
    square_theta += 5.0;
    glutPostRedisplay();
  }
  else if (key == 'd') {
    square_theta -= 5.0;
    glutPostRedisplay();
  }
  else if (key == 'z') {
    square_scale -= 0.1;
    glutPostRedisplay();
  }
  else if (key == 'c') {
    square_scale += 0.1;
    glutPostRedisplay();
  }
  else if (key == 'q') {
    exit(1);
  }
}

//----------------------------------------------------------------------------

// drag -- mouse motion with button down

void motion(int x, int y)
{
  square_x = x;
  square_y = win_h-y;

  glutPostRedisplay();
}

//----------------------------------------------------------------------------

// click -- mouse button pressed or released

void mouse(int button, int state, int x, int y)
{
  if (button == GLUT_LEFT_BUTTON) {

    square_x = x;
    square_y = win_h-y;

    // catch...

    if (state == GLUT_DOWN) {
      last_square_dx = square_dx;
      square_dx = 0;
    }

    // ...and release

    else if (state == GLUT_UP)
      square_dx = last_square_dx;

    glutPostRedisplay();
  }  
}

//----------------------------------------------------------------------------

void init (void) 
{
  printf("Click and drag to reposition square\n");
  printf("Press a/d to rotate\n");
  printf("Press z/c to scale\n");
  printf("q to quit\n");

  glClearColor (0.0, 0.0, 0.0, 0.0);
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0.0, win_w, 0.0, win_h);

  int numTextures = 1;
  char **texfilename = (char **) malloc(numTextures * sizeof(char *));
  for (int i = 0; i < numTextures; i++) 
    texfilename[i] = (char *) malloc(256 * sizeof(char));

  sprintf(texfilename[0], "saucer.ppm");
  load_sprites(numTextures, texfilename);

  // width, height (in pixels), color of underlying polygon, texture id
  sprite = new Sprite(50, 50, 1.0, 1.0, 1.0, 1);
}

//----------------------------------------------------------------------------

void idle()
{
  // reverse direction?

  if ((square_dx < 0 && square_x < 0)  || // traveling left and leaving left edge
      (square_dx > 0 && square_x >= win_w))
    square_dx = -square_dx;

  // move

  square_x += square_dx;

  // draw

  glutPostRedisplay();
}

//----------------------------------------------------------------------------

int main(int argc, char** argv)
{
  glutInit(&argc, argv);
  glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);
  glutInitWindowSize (win_w, win_h); 
  glutInitWindowPosition (100, 100);
  glutCreateWindow ("hello");
  init();
  
  glutIdleFunc(idle);
  glutMouseFunc(mouse);
  glutKeyboardFunc(keyboard);
  glutMotionFunc(motion);
  glutDisplayFunc(display); 

  glutMainLoop();

  return 0;  
}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
